#!/usr/bin/perl -w
##############################################################################
# Prepare data about the given application to be inserted into the database.
#
# This script takes a list of components to be processed from file and
# calls 'componenttodb.pl' for each file. The results are collected in
# one file that can be used by 'upload_app_data.pl' for adding information
# to the database.
#
# Author: Denis Silakov <silakov@ispras.ru>
##############################################################################

use strict;
use Getopt::Long;

use Text::Wrap;
Getopt::Long::Configure ("posix_default", "no_ignore_case");

use Distros;
# Application information
my $AppName="";
my $AppVer="";
my $ComponentsFile;
my $RpmFile;
my $RpmFileList;
my $AppFilesList;
my $Comment='';
my $Aarch="";
my $OutputFile;
my $Packager="";
my $URL="";
my $LicenseType="";
my $AppUI="";

# Package manager used in the distribution
my $PackageManager;

sub HELP_MESSAGE();
sub guess_platform($ );
sub get_app_version();
sub get_app_comment();
sub get_app_packager();
sub get_app_url();
sub get_app_license();

# options
my $opt_help;
my $quiet;

GetOptions("help|h!" => \$opt_help,
            "comment|c=s" => \$Comment,
            "outfile|o=s" => \$OutputFile,
            "arch|a=s" => \$Aarch,
            "version|v=s" => \$AppVer,
            "licensetype|t=s" => \$LicenseType,
            "ui=s" => \$AppUI,
            "infile|i=s" => \$ComponentsFile,
            "rpm|r=s" => \$RpmFile,
            "rpmlist|s=s" => \$RpmFileList,
            "filelist|l=s" => \$AppFilesList,
            "packagemanager|m=s" => \$PackageManager,
            "packager|p=s" => \$Packager,
            "url|u=s" => \$URL,
            "quiet|q!" => \$quiet);

HELP_MESSAGE() if (defined $opt_help);

if(@ARGV != 1) {HELP_MESSAGE(); exit(1);}

# Guess package manager, if it was not specified
if( (not $PackageManager) and (not $AppFilesList) and (not $RpmFile) ) {
    $PackageManager = guess_package_manager();
    if( $PackageManager eq "unknown" ) {
        print STDERR "Cannot guess package manager! Please specify it directly using '-m' option.";
        exit(1);
    }
    else {
        print STDERR "No package manager was specified, guessing it is $PackageManager\n";
    }
}

$AppName = $ARGV[0];

if( ($RpmFile or $RpmFileList) and !does_shell_know("rpm") ) {
	print STDERR "You are trying to process rpm file(s), but you don't have rpm installed in your system.";
	exit(1);
}

if( ($RpmFile or $RpmFileList) and !does_shell_know("rpm2cpio") ) {
	print STDERR "You are trying to process rpm file(s), but you don't have rpm2cpio installed in your system.";
	exit(1);
}

if( !$Packager and !$RpmFile and !$RpmFileList ) {
    if( $AppFilesList or $ComponentsFile ) {
        print STDERR "You should specify packager manually when you use files or components list.\n";
	}
	else {
		get_app_packager();
	}
}

if( !$URL ) {
    if( $AppFilesList or $ComponentsFile or $RpmFileList ) {
        print STDERR "You should specify packager manually when you use files or components list.\n";
	}
	else {
		get_app_url();
	}
}

if( !$LicenseType ) {
    if( $AppFilesList or $ComponentsFile or $RpmFileList ) {
        print STDERR "You should specify license type manually when you use files or components list.\n";
	}
	else {
		get_app_license();
	}
}

if( $AppUI and ($AppUI ne "GUI") and ($AppUI ne "non-GUI") ) {
	$AppUI = "";
}

if( !$AppVer ) {
    # Guess package manager, if it was not specified
    if( $AppFilesList or $ComponentsFile or $RpmFileList ) {
        print STDERR "You should specify version manually when you use files or components list.\n";
    }
    else {
        get_app_version();
    }
}

if( not $Aarch ) {
    $Aarch = guess_platform(`uname -m`);
}

if( not $OutputFile ) {
    $OutputFile = $AppName."_".$AppVer."_".$Aarch."_upload_data";
}

##############################################################################
# Main section
##############################################################################

if( !$Comment ) {
    # Guess package manager, if it was not specified
    if( $AppFilesList or $RpmFileList ) {
        print STDERR "If you want to add summary for you application, specify it with '-c' option.\n";
    }
    else {
        get_app_comment();
    }
}

system "echo \"!Application '$AppName' '$AppVer' '$Aarch' '$Packager' '$URL' '$Comment'\" > \"$OutputFile\"";
system "echo \"!AppCategory '$LicenseType' '$AppUI'\" >> \"$OutputFile\"";

my $options = "";
if( $quiet ) {
    $options.= " -q ";
}
if( $RpmFile ) {
	system "rm -rf /var/tmp/apptodb; mkdir /var/tmp/apptodb; cp \"$RpmFile\" /var/tmp/apptodb/";
	my $cpio = $quiet ? "cpio -id --quiet" : "cpio -id";
	system "cd /var/tmp/apptodb/ && rpm2cpio \"$RpmFile\" | $cpio ";
	system "rm \"/var/tmp/apptodb/$RpmFile\"";
	system "echo \"/var/tmp/apptodb/\" > /var/tmp/apptodb/apptodb_list";
    $options.= " -l /var/tmp/apptodb/apptodb_list";
}
elsif( $RpmFileList ) {
	system "rm -rf /var/tmp/apptodb; mkdir /var/tmp/apptodb";
	my $cpio = $quiet ? "cpio -id --quiet" : "cpio -id";

    open( RPM_LIST, "<$RpmFileList" );
    while( <RPM_LIST> )
    {
        chomp;
        if( $_ ) {
			system "cp \"$_\" /var/tmp/apptodb/ ";
			system "cd /var/tmp/apptodb/ && rpm2cpio \"$_\" | $cpio ";
			system "rm \"/var/tmp/apptodb/$_\"";
		}
    }
    close( RPM_LIST );

	system "echo \"/var/tmp/apptodb/\" > /var/tmp/apptodb/apptodb_list";
    $options.= " -l /var/tmp/apptodb/apptodb_list";
}
elsif( $AppFilesList ) {
    $AppFilesList =~ s/\"/\\\"/g;
    $options.= " -l \"$AppFilesList\" ";
}
if( $PackageManager ) {
    $options.= " -m $PackageManager ";
}

if( $ComponentsFile ) {
    open( COMPONENT_LIST, "<$ComponentsFile" );
    while( <COMPONENT_LIST> )
    {
        chomp;
		# in the input file we can find either the component name
		# and the package that provide it or only the component name
		if( /((\w|\:|\-|\.|\d)+) (.+)/) {
			system "./componenttodb.pl $options -c \"$1\" \"$3\"";
			system "cat \"".$3."_upload_data\" >> \"$OutputFile\"";
			system "rm \"".$3."_upload_data\"";
		}
		elsif( $_ ) {
			system "./componenttodb.pl $options \"$_\"";
			system "cat \"".$_."_upload_data\" >> \"$OutputFile\"";
			system "rm \"".$_."_upload_data\"";
		}
    }
}
else {
    # No file with packages
    system "./componenttodb.pl $options \"$AppName\"";
    system "cat \"".$AppName."_upload_data\" >> \"$OutputFile\"";
    system "rm \"".$AppName."_upload_data\"";

	# Delete "/var/tmp/apptodb" from result file if we are processing rpms
    if( $RpmFile or $RpmFileList ) {
    	my $tmpFile = $OutputFile.".tmp";
    	system "cat \"$OutputFile\" | sed s/\\\\/var\\\\/tmp\\\\/apptodb// > \"$tmpFile\"";
    	system "mv \"$tmpFile\" \"$OutputFile\"";
    }
}

if( $RpmFile ) {
	system "rm -rf /var/tmp/apptodb";
}

##############################################################################
# Miscellaneous functions - obtain different application information
##############################################################################

sub get_app_packager()
{
	my $PkgLine;

    if( $PackageManager eq "rpm" ) {
    	# 'Distribution' field (if any) is usually the most informative
		$PkgLine = `rpm -q --info $AppName | LC_ALL=C grep Distribution | cut -d\\: -f2- `;

		if( !$PkgLine or $PkgLine =~ '\(none\)' ) {
			$PkgLine = `rpm -q --info $AppName | LC_ALL=C grep Vendor | cut -d\\: -f3- `;
		}

		# 'Packager' field can contain email embreced with '<>' - cut it
		if( !$PkgLine or $PkgLine =~ '\(none\)' ) {
			$PkgLine = `rpm -q --info $AppName | LC_ALL=C grep Packager | cut -d\\: -f2- | cut -d\\< -f1`;
		}

		$Packager = trim($PkgLine);
		if( !$Packager ) {
            print STDERR "Can't detect application packager!\n";
		}
    }
    elsif( $PackageManager eq "dpkg" ) {
    	# Actually, two 'Maintainer' fields can exist - 'Maintainer' and 'Original-Maintainer'
    	# The former will be printed first and this is what we need. Otherwise 'Original-Maintainer'
    	# is our only choise.
		$PkgLine = `dpkg -s $AppName | LC_ALL=C grep Maintainer | cut -d\\: -f2- | cut -d\\< -f1`;
		$Packager = trim($PkgLine);
		if( !$Packager ) {
            print STDERR "Can't detect application packager!\n";
		}
    }
    elsif( $PackageManager eq "emerge" ) {
    	$Packager = 'Gentoo';
    }
    else {
        print STDERR "Can't detect application packager - package manager '$PackageManager' is not supproted\n";
    }
}

sub get_app_url()
{
	my $URLLine;

	if( $RpmFile ) {
		$URLLine = `rpm -qivp $RpmFile | LC_ALL=C grep URL | cut -d\\: -f2- `;
		$URL = trim($URLLine);
		if( !$URL ) {
            print STDERR "Can't detect application url!\n";
		}

		return;
	}

    if( $PackageManager eq "rpm" ) {
		$URLLine = `rpm -q --info $AppName | LC_ALL=C grep URL | cut -d\\: -f2- `;
		$URL = trim($URLLine);
		if( !$URL ) {
            print STDERR "Can't detect application url!\n";
		}
    }
    elsif( $PackageManager eq "dpkg" ) {
        print STDERR "Can't detect application url - package manager '$PackageManager' is not supproted yet\n";
    }
    elsif( $PackageManager eq "emerge" ) {
		my $ebuild_str = "";
		$ebuild_str = `equery which $AppName`;
		if( $ebuild_str ) {
			chomp $ebuild_str;
			my $url_str = "";
			$url_str = `cat $ebuild_str | LC_ALL=C grep HOMEPAGE`;
			if( $url_str =~ /HOMEPAGE\=\"([^\"]+)\"/ ) {
				$URL = $1;
			}
			else {
				print STDERR "Can't detect application url - invalid link in ebuild\n";
			}
		}
		else {
				print STDERR "Can't detect application url - something wrong with ebuild string (or maybe you don't have 'equery' installed?\n";
		}
    }
    else {
        print STDERR "Can't detect application url - package manager '$PackageManager' is not supproted\n";
    }
}

sub guess_license_type($ )
{
	my $License = $_[0];

	if( $License =~ /GNU/ or $License =~ /GPL/ or $License =~ /BSD/ or $License =~ /MIT/ or $License =~ /MPL/
			or $License =~ /Public Domain/ or $License =~ /Apache Software License/ ) {
		$LicenseType="Open Source";
	}
}

sub get_app_license()
{
	my $LicenseLine;
	my $License;

	if( $RpmFile ) {
		$LicenseLine = `rpm -qivp $RpmFile | LC_ALL=C grep License | cut -d\\: -f3- `;
		$License = trim($LicenseLine);
		if( !$License ) {
            print STDERR "Can't detect application license type!\n";
		}

		guess_license_type($License);
		return;
	}

    if( $PackageManager eq "rpm" ) {
		$LicenseLine = `rpm -q --info $AppName | LC_ALL=C grep License | cut -d\\: -f3- `;
		$License = trim($LicenseLine);
		if( !$License ) {
            print STDERR "Can't detect application license!\n";
		}
    }
    elsif( $PackageManager eq "dpkg" ) {
        print STDERR "Can't detect application license - package manager '$PackageManager' is not supproted yet\n";
    }
    elsif( $PackageManager eq "emerge" ) {
        print STDERR "Can't detect application license - package manager '$PackageManager' is not supproted yet\n";
# 		my $ebuild_str = "";
# 		$ebuild_str = `equery which $AppName`;
# 		if( $ebuild_str ) {
# 			chomp $ebuild_str;
# 			my $url_str = "";
# 			$url_str = `cat $ebuild_str | LC_ALL=C grep HOMEPAGE`;
# 			if( $url_str =~ /HOMEPAGE\=\"([^\"]+)\"/ ) {
# 				$URL = $1;
# 			}
# 			else {
# 				print STDERR "Can't detect application url - invalid link in ebuild\n";
# 			}
# 		}
# 		else {
# 				print STDERR "Can't detect application url - something wrong with ebuild string (or maybe you don't have 'equery' installed?\n";
# 		}
    }
    else {
        print STDERR "Can't detect application url - package manager '$PackageManager' is not supproted\n";
    }

	guess_license_type($License);
}

sub get_app_version()
{
    my $VerLine;

	if( $RpmFile ) {
        $VerLine = `rpm -qivp $RpmFile | LC_ALL=C grep Version | LC_ALL=C grep Vendor`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s+\:\s+((\w|\.|\-|\_|\d)+)\s+Vendor/ ) {
            $AppVer = $1;
        }
        else {
            print STDERR "Can't detect application version!\n";
        }

        return;
	}

    if( $PackageManager eq "rpm" ) {
       	$VerLine = `rpm -qi $AppName | LC_ALL=C grep Version | LC_ALL=C grep Vendor`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s+\:\s+((\w|\.|\-|\_|\d)+)\s+Vendor/ ) {
            $AppVer = $1;
        }
        else {
            print STDERR "Can't detect application version!\n";
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        $VerLine = `dpkg -s $AppName | LC_ALL=C grep -i version`;
        chomp($VerLine);
		if( $VerLine =~ /(V|v)ersion\s*:\s*(\S+)/ ) {
            $AppVer = $2;
        }
        else {
            print STDERR "Can't detect application version!\n";
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        $VerLine = `emerge --pretend --nodeps $AppName | LC_ALL=C grep $AppName`;
         chomp($VerLine);
        if( $VerLine =~ /\[(\s|\w|\d)*\](\s|\w|\/|\d|\.|\-|\+)*\[((\s|\w|\d|\.|\-|\+)*)\]/ ) {
            $AppVer = $3;
        }
        elsif( $VerLine =~ /no\ ebuilds/ ) {
            print STDERR "Can't detect application version - it seems that it is not installed\n";
        }
        elsif( $VerLine =~ /$AppName(\s|\-)*((\w|\.|\d|\-|\+)+)/ ) {
            $AppVer = $2;
        }
        else {
            print STDERR "Can't detect application version!\n";
        }
    }
    else {
        print STDERR "Can't detect application version - package manager '$PackageManager' is not supproted\n";
    }
}

sub get_app_comment()
{
    my $CommentLine;

	if( $RpmFile ) {
        $CommentLine = `rpm -qivp $RpmFile | LC_ALL=C grep Summary | sed s/\\'/\\'\\^/g`;
        if( $CommentLine =~ /Summary\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            print STDERR "Can't get application summary!\n";
        }
        return;
    }

    if( $PackageManager eq "rpm" ) {
        $CommentLine = `rpm -qi $AppName | LC_ALL=C grep Summary | sed s/\\'/\\'\\^/g`;
        if( $CommentLine =~ /Summary\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            print STDERR "Can't get application summary!\n";
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        $CommentLine = `dpkg -s $AppName | LC_ALL=C grep Description | sed s/\\'/\\'\\^/g`;
        if( $CommentLine =~ /Description\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            print STDERR "Can't get application summary!\n";
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        if( does_shell_know("equery") ) {
    	    my $ebuild_str = "";
            $ebuild_str = `equery which $AppName`;
	    if( $ebuild_str ) {
    	            chomp $ebuild_str;
        	    my $comment_str = "";
            	    $comment_str = `cat $ebuild_str | LC_ALL=C grep DESCRIPTION | sed s/\\'/\\'\\^/g`;
            	    if( $comment_str =~ /DESCRIPTION\=\"([^\"]+)\"/ ) {
                    	    $Comment = $1;
            	    }
            	    else {
            	            print STDERR "Can't detect application url - invalid link in ebuild\n";
            	    }
    	    }
    	    else {
                print STDERR "Can't get application summary - something wrong with ebuild name\n";
    	    }
        }
        else {
            print STDERR "Can't get application summary - 'equery' tool from 'gentoolkit' package is required.\n";
        }
    }
    else {
        print STDERR "Can't get application summary - package manager '$PackageManager' is not supproted\n";
    }
}

sub HELP_MESSAGE()
{
print STDERR <<"EOM"

$0: Obtain information about application.

Usage: $0 [options] app_name

app_name                      Name of the application

-a|--arch                     Specify architecture. Possible values are IA32, IA64, PPC32,
                              PPC64, S390, S390X and x86-64. By default the output of `uname -m`
                              is analyzied
-h|--help                     Display this help message
-v|--version app_version      Application version. If none is specified, the script will try to obtain it from package manager
-i|--infile filename          File with the list of packages to be processed (i.e. packages that form the application)
                              If no file is specified, application name is considered to be the only package
-l|--filelist filename        File containing list of file names that are installed by the application.
                              (Directory names are also allowed; all files from given directories will be processed)
-r|--rpm filename             RPM file with application to be processed. Use this option if you don't want to
                              install the application and the whole application is provided by single rpm file.
-s|--rpmlist filename         List of RPM files containing application binaries. Use this option if you don't want to
                              install the application and the application is provided by several rpm files.
-o|--outfile filename         Filename where the data for upload will be situated
                              By default, "<app_name>_<app_ver>_<arch>_upload_data" is used
-q|--quiet                    Do not print any information messages
-u|--url URL                  Application homepage URL
-t|--licensetype type         License type. "Open Source", "Proprietary" and "Mixed"  are valid values at the moment.
-p|--packager packager        Application packager - any information about uploaded application build
                              (distribution, maintainer, etc.). This can be useful since different builds of the same
                              application may require different sets of external interfaces. If nothing is specified,
                              then the script will try to guess it using package manager
-m|--packagemanager pkg_name  Package manager used in the distribution. Supported values
                              are 'rpm', 'dpkg' and 'emerge'. If none is specified, the script
                              will try to guess it
--ui ui_kind                  User interface style. Possible values are 'non-GUI' and 'GUI'
-c|--comment comment_text     Comment for the application (will be inserted into the database). If none is specified, the
                              script will try to obtain it from package manager

EOM
      ; exit(1);
}
